<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_interactive_maps\Entity\InteractiveMap;
use Drupal\simple_interactive_maps\MapDataLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Interactive Map form.
 */
final class InteractiveMapForm extends EntityForm {

  /**
   * Form Constructor.
   *
   * @param \Drupal\simple_interactive_maps\MapDataLoader $mapDataLoader
   *   Data loader service.
   *
   * @todo see if this is needed or needs to be replaced with entity storage.
   */
  public function __construct(protected MapDataLoader $mapDataLoader) {}

  public static function create(ContainerInterface $container): InteractiveMapForm {
    /** @var \Drupal\simple_interactive_maps\MapDataLoader $mapDataLoader */
    $mapDataLoader = $container->get('simple_interactive_maps.map_data_loader');
    return new InteractiveMapForm($mapDataLoader);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $config = $this->config('simple_interactive_maps.settings');

    $form = parent::form($form, $form_state);

    assert($this->entity instanceof InteractiveMap);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [InteractiveMap::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    // Generate a rich text editor for the description field.
    $form['description'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description')['value'] ?? '',
      '#format' => $this->entity->get('description')['format'] ?? 'full_html',
    ];

    // Map default fill color.
    $form['fill_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Fill Color'),
      '#default_value' => $this->entity->get('fillColor') ?? $config->get('default_fill_color') ?? '#4ba0a6',
      '#description' => $this->t('The default fill color for the map.'),
    ];

    $form['hover_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Hover Color'),
      '#default_value' => $this->entity->get('hoverColor') ?? $config->get('default_hover_color') ?? '#076369',
      '#description' => $this->t('The color of the map when hovered over.'),
    ];

    // Map default stroke color.
    $form['stroke_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Stroke Color'),
      '#default_value' => $this->entity->get('strokeColor') ?? $config->get('default_stroke_color') ?? '#e6e6e6',
      '#description' => $this->t('The default stroke color for the map.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {

    assert($this->entity instanceof InteractiveMap);

    if ($this->entity->isNew()) {
      try {
        $stateData = $this->mapDataLoader->loadMapData();
      }
      catch (\JsonException $e) {
        throw new HttpException(500, 'Failed to load map data.');
      }

      $config = $this->config('simple_interactive_maps.settings');

      $details = [];

      foreach ($stateData as $key => $value) {
        $details[$key] = [
          'id' => $key,
          'label' => $value['label'],
          'tooltip' => $config->get('default_state_tooltip') ?? ['value' => '', 'format' => 'full_html'],
          'url' => '',
          'hidden' => 0,
          'fill_color' => $form_state->get('fill_color') ?? $config->get('default_fill_color') ?? '#4ba0a6',
          'hover_color' => $form_state->get('hover_color') ?? $config->get('default_hover_color') ?? '#076369',
          'stroke_color' => $form_state->get('stroke_color') ?? $config->get('default_stroke_color') ?? '#e6e6e6',
          'action' => 'none',
          'content' => ['value' => '', 'format' => 'full_html'],
          'region' => '',
        ];
        $this->entity->set($key, $value);
      }
      $this->entity->set('regions', []);
      $this->entity->set('details', $details);
    }
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        default => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
