<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\TempStoreException;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class MapDetailsForm extends FormBase {

  use InteractiveMapFormTempStoreTrait;

  public function __construct(protected ConfigEntityStorageInterface $mapStorage, protected FormBuilderInterface $formBuilder) {}

  public static function create(ContainerInterface $container): MapDetailsForm {

    /** @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface $mapStorage */
    $mapStorage = $container->get('entity_type.manager')
      ->getStorage('interactive_map');

    /** @var \Drupal\Core\Form\FormBuilderInterface $formBuilder */
    $formBuilder = $container->get('form_builder');

    return new MapDetailsForm($mapStorage, $formBuilder);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_map_details';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL): array {
    if ($interactive_map === NULL) {
      throw new HttpException(404, 'Map not found.');
    }

    $stateDetails = $interactive_map->get('details') ?: [];

    $detailsStorage = $this->getMapTempStore($interactive_map);
    if ($detailsStorage->getMetadata(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY) === NULL) {
      try {
        $detailsStorage->set(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY, $stateDetails);
      }
      catch (TempStoreException) {
        throw new HttpException(500, 'Failed to store state details.');
      }
    }

    $form['id'] = [
      '#type' => 'hidden',
      '#value' => $interactive_map->id(),
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ],
    ];

    $form['states'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('State'),
        $this->t('region'),
        $this->t('Edit'),
      ],
    ];

    foreach ($stateDetails as $id => $state) {
      $form['states'][$id] = [
        '#attributes' => ['class' => ['ed-state-form-row'], 'id' => ['row-state-' . strtolower($id)]],
      ];

      $form['states'][$id]['label'] = [
        '#type' => 'item',
        '#markup' => $state['label'],
      ];

      $regions = $interactive_map->get('regions') ?: [];

      $form['states'][$id]['region'] = [
        '#type' => 'item',
        '#markup' => $state['region'] ? $regions[$state['region']]['label'] : '',
      ];

      $form['states'][$id]['edit-link'] = [
        '#type' => 'link',
        '#title' => $this->t('Edit'),
        '#url' => Url::fromRoute('simple_interactive_maps.map_details_modal_form', [
          'interactive_map' => $interactive_map->id(),
          'stateId' => $id,
        ]),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => \json_encode([
            'width' => 800,
            'minHeight' => 650,
            'title' => $this->t('Editing @state Details', ['@state' => $state['label']]),
          ], JSON_THROW_ON_ERROR),
        ],
      ];
    }

    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $map = $this->mapStorage->load($form_state->getValue('id'));

    if ($map instanceof InteractiveMapInterface) {
      $detailsStorage = $this->getMapTempStore($map);
      $stateDetails = $detailsStorage->get(MapDataloader::MAP_TEMP_STORE_DETAILS_KEY);
      $map->set('details', $stateDetails);
      $map->save();
      $this->messenger()->addStatus($this->t('The map has been updated.'));
      $detailsStorage->delete(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY);
    }
    else {
      $this->messenger()->addError($this->t('Failed to update the map.'));
    }

    $route_name = $this->getRouteMatch()->getRouteName();
    $route_parameters = $this->getRouteMatch()->getRawParameters()->all();
    $url = Url::fromRoute($route_name, $route_parameters);
    $form_state->setRedirectUrl($url);
  }

}
