<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\PrivateTempStore;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Drupal\simple_interactive_maps\RegionTableGeneratorTrait;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class MapRegionForm extends FormBase {

  use RegionTableGeneratorTrait, InteractiveMapFormTempStoreTrait;

  /**
   * Region data temporary storage.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  public PrivateTempStore $regionTempStore;

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_map_region';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL): array {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(404, 'Interactive map not found.');
    }

    $tempStore = $this->getMapTempStore($interactive_map);

    // Load region data from the tempstore.
    if ($tempStore->getMetadata(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY) !== NULL) {
      $regions = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
      if (!$interactive_map->compareRegionsData($regions)) {
        $this->messenger()->addWarning('You have unsaved changes.', FALSE);
      }

    }
    else {
      // No data in tempstore, new session, init from map config.
      $regions = $interactive_map->get('regions');
      $tempStore->set(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY, $regions);
    }

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['add_region'] = [
      '#type' => 'link',
      '#title' => $this->t('Add region'),
      '#url' => Url::fromRoute('simple_interactive_maps.map_region_add_form', ['interactive_map' => $interactive_map->id()]),
      '#attributes' => [
        'class' => [
          'button',
          'button--primary',
          'use-ajax',
        ],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => \json_encode([
          'width' => 1000,
          'minHeight' => 600,
        ]),
      ],
    ];

    $form['regions'] = $this->generateRegionsTable($regions, $interactive_map);

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $interactive_map = $form_state->getValue('map');
    $mapTempStore = $this->getMapTempStore($interactive_map);
    $regions = $mapTempStore->get(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
    $interactive_map->set('regions', $regions);
    $interactive_map->save();
    $mapTempStore->delete(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);

    $route_name = $this->getRouteMatch()->getRouteName();
    $route_parameters = $this->getRouteMatch()->getRawParameters()->all();
    $url = Url::fromRoute($route_name, $route_parameters);
    $this->messenger()->addStatus('Region data saved.');
    $form_state->setRedirectUrl($url);
  }

}
