<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\Entity\InteractiveMap;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\MapDataLoader;
use Drupal\simple_interactive_maps\RegionTableGeneratorTrait;

/**
 * @todo Add a description for the form.
 */
final class RegionDeleteConfirmForm extends ConfirmFormBase {

  use InteractiveMapFormTempStoreTrait, RegionTableGeneratorTrait;

  /**
   * The map that the region will be deleted from.
   *
   * @var \Drupal\simple_interactive_maps\Entity\InteractiveMap|null
   */
  protected ?InteractiveMap $map;

  /**
   * The region to be deleted.
   *
   * @var string|null
   */
  protected ?string $region;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_region_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to do delete this region?', [
      '@region' => $this->region,
      '@map' => $this->map->label(),
    ]);
  }

  public function getDescription() {
    return $this->t('This will permanently delete the region from this map and cannot be undone.');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('system.admin_config');
  }

  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMap $interactive_map = NULL, string $region = NULL): array {
    $this->map = $interactive_map;
    $this->region = $region;

    $form = parent::buildForm($form, $form_state);

    $form['actions']['submit']['#ajax'] = [
      'callback' => '::submitForm',
    ];

    $form['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#ajax' => [
        'callback' => '::cancelAjax',
      ],
    ];

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['region'] = [
      '#type' => 'value',
      '#value' => $region,
    ];

    return $form;
  }

  public function cancelAjax(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();

    // Close the modal dialog
    $response->addCommand(new CloseModalDialogCommand());

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();

    $tempStore = $this->getMapTempStore($this->map);
    $regions = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
    unset($regions[$this->region]);
    $tempStore->set(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY, $regions);

    // Close the modal dialog
    $response->addCommand(new CloseModalDialogCommand());

    // Generate the updated regions table
    $mapRegionForm = new MapRegionForm();
    $updatedRegionsTable = $mapRegionForm->generateRegionsTable($regions, $this->map);

    // Replace the old regions table with the updated one
    $response->addCommand(new ReplaceCommand('#ed-regions-table', $updatedRegionsTable));
    $response->addCommand(new MessageCommand('You have unsaved changes.', 'warning', ['sticky' => TRUE], TRUE));
    $response->addCommand(new MessageCommand('The region has been deleted from the map.'));

    return $response;
  }

}
