<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ChangedCommand;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\TempStoreException;
use Drupal\simple_interactive_maps\EdMapsLoggerTrait;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class StateModalForm extends FormBase {

  use InteractiveMapFormTempStoreTrait, EdMapsLoggerTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_state_modal';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL, string $stateId = NULL): array {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(500, 'Interactive map not found.');
    }

    $detailsStorage = $this->getMapTempStore($interactive_map);
    $stateData = $detailsStorage->get(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY);
    $state = $stateData[$stateId];

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['id'] = [
      '#type' => 'hidden',
      '#value' => $stateId,
    ];

    $form['label'] = [
      '#type' => 'value',
      '#value' => $state['label'],
    ];

    $form['state'] = [
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    // Create a tab group
    $form['state']['tabs'] = [
      '#type' => 'horizontal_tabs',
      '#default_tab' => 'edit-general',
    ];

    $form['state']['tabs']['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-general']],
    ];

    $form['state']['tabs']['general']['tooltip'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Tooltip text'),
      '#default_value' => $state['tooltip']['value'] ?? '',
      '#format' => $state['tooltip']['format'] ?? 'full_html',
    ];

    $form['state']['tabs']['general']['hidden'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hidden'),
      '#default_value' => $state['hidden'] ?: FALSE,
    ];

    $regions = $interactive_map->get('regions');

    $regionOptions = [];
    $regionActive = FALSE;
    if (!empty($regions)) {
      foreach ($regions as $regionId => $region) {
        $regionOptions[$regionId] = $region['label'];
      }

      $regionActive = TRUE;
    }

    $form['state']['tabs']['general']['region'] = [
      '#type' => 'select',
      '#title' => $this->t('Region'),
      '#options' => $regionOptions,
      '#empty_option' => '- None -',
      '#empty_value' => '',
      '#default_value' => $state['region'] ?? '',
      '#disabled' => !$regionActive,
    ];

    $form['state']['tabs']['color'] = [
      '#type' => 'details',
      '#title' => $this->t('Colors'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-color']],
    ];

    $form['state']['tabs']['color']['fill_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Fill Color'),
      '#default_value' => $state['fill_color'] ?? '#4ba0a6',
      '#description' => $this->t('The color to fill the state with.'),
    ];

    $form['state']['tabs']['color']['hover_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Hover color'),
      '#default_value' => $state['hover_color'] ?? '#076369',
      '#description' => $this->t('The color to fill the state with when hovered over.'),
    ];

    $form['state']['tabs']['color']['stroke_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Stroke color'),
      '#default_value' => $state['stroke_color'] ?? '#076369',
      '#description' => $this->t('The color to fill the state with when hovered over.'),
    ];

    $form['state']['tabs']['behavior'] = [
      '#type' => 'details',
      '#title' => $this->t('Behaviors'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-behaviors']],
    ];

    $form['state']['tabs']['behavior']['action'] = [
      '#type' => 'radios',
      '#title' => $this->t('Action'),
      '#options' => [
        'none' => $this->t('None'),
        'url' => $this->t('Navigate to URL'),
        'content' => $this->t('Display Content'),
      ],
      '#default_value' => $state['action'] ?? 'none',
    ];

    $form['state']['tabs']['behavior']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Target URL'),
      '#default_value' => $state['url'] ?? '',
      '#states' => [
        'visible' => [
          ':input[name="state[tabs][behavior][action]"]' => ['value' => 'url'],
        ],
      ],
    ];

    $form['state']['tabs']['behavior']['content'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Content'),
      '#default_value' => $state['content']['value'] ?? '',
      '#format' => $state['content']['format'] ?? 'full_html',
      '#states' => [
        'visible' => [
          ':input[name="state[tabs][behavior][action]"]' => ['value' => 'content'],
        ],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save State Info'),
        '#button_type' => 'primary',
        '#ajax' => [
          'callback' => '::submitForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Saving State Info...'),
          ],
          'prevent' => 'submit',
        ],
      ],
      'cancel' => [
        '#type' => 'button',
        '#value' => $this->t('Cancel'),
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => '::closeModalForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Closing Form...'),
          ],
          'prevent' => 'submit',
        ],
      ],
    ];

    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $detailsStorage = $this->getMapTempStore($form_state->getValue('map'));

    $stateDetails = $detailsStorage->get(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY);
    $stateId = $form_state->getValue('id');
    $newStateValues = [
      'id' => $form_state->getValue('id'),
      'label' => $form_state->getValue('label'),
      'hidden' => $form_state->getValue('state')['tabs']['general']['hidden'],
      'region' => $form_state->getValue('state')['tabs']['general']['region'],
      'fill_color' => $form_state->getValue('state')['tabs']['color']['fill_color'],
      'hover_color' => $form_state->getValue('state')['tabs']['color']['hover_color'],
      'stroke_color' => $form_state->getValue('state')['tabs']['color']['stroke_color'],
      'tooltip' => $form_state->getValue('state')['tabs']['general']['tooltip'],
      'action' => $form_state->getValue('state')['tabs']['behavior']['action'],
      'url' => $form_state->getValue('state')['tabs']['behavior']['url'],
      'content' => $form_state->getValue('state')['tabs']['behavior']['content'],
    ];
    $stateDetails[$stateId] = $newStateValues;

    try {
      $detailsStorage->set(MapDataLoader::MAP_TEMP_STORE_DETAILS_KEY, $stateDetails);
    }
    catch (TempStoreException $e) {
      $this->getMapsLogger()->error($e->getMessage());
    }

    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand(FALSE));

    $rowSelector = sprintf('#row-state-%s', strtolower($stateId));
    $selector = sprintf('#edit-states-%s-label', strtolower($stateId));
    $response->addCommand(new ChangedCommand($rowSelector, $selector));
    $response->addCommand(new MessageCommand('State details for ' . $newStateValues['label'] . ' saved.', NULL, ['type' => 'status'], TRUE));
    return $response;
  }

  public function closeModalForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

}
