<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class WordPressDataImportForm extends DataImportFormBase {

  private array $states = [];

  private array $regions = [];

  private array $regionMap = [];

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_word_press_data_import';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL): array {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(404, 'Interactive map not found');
    }

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['wordpress_data_file'] = [
      '#type' => 'file',
      '#title' => $this->t('WordPress Data Export File'),
      '#element_validate' => ['::validateFileUpload'],
      '#description' => $this->t('Upload a CSV file with WordPress data.'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Upload'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $fieldId = $this->getUploadFieldId();
    $file = $form_state->getValue($fieldId);
    $file_path = $file->getFileUri();

    $file = fopen($file_path, 'rb');

    while (($line = fgetcsv($file)) !== FALSE) {
      if ($line[0] === 'id' && count($line) === 16) {
        $mapper = 'mapDataToState';
      }
      elseif ($line[0] === 'groupId' && count($line) === 16) {
        $mapper = 'mapDataToRegion';
      }
      else {
        if (isset($mapper)) {
          $this->$mapper($line);
        }
      }
    }

    // Reconcile the region ids in the states data.
    foreach ($this->states as &$state) {
      if (isset($this->regionMap[$state['region']])) {
        $state['region'] = $this->regionMap[$state['region']];
      }
      else {
        $state['region'] = '';
      }
    }

    $interactiveMap = $form_state->getValue('map');

    $interactiveMap->set('details', $this->states);
    $interactiveMap->set('regions', $this->regions);

    $interactiveMap->save();

    $this->messenger()->addStatus($this->t('The map data has been imported.'));
    $form_state->setRedirect('simple_interactive_maps.simple_interactive_maps_tools', ['interactive_map' => $interactiveMap->id()]);
  }

  protected function mapDataToState(array $row): void {

    $config = $this->config('simple_interactive_maps.settings');

    $action = match($row[8]) {
      'info' => 'content',
      'link' => 'url',
      default => 'none',
    };

    $state = [
      'id' => $row[1],
      'label' => $row[2],
      'hidden' => (bool) $row[14],
      'region' => $row[15],
      'fill_color' => $row[10],
      'hover_color' => $row[11],
      'stroke_color' => $config->get('default_stroke_color'),
      'tooltip' => [
        'value' => $config->get('default_state_tooltip')['value'],
        'format' => $config->get('default_state_tooltip')['format'],
      ],
      'action' => $action,
      'url' => $row[6],
      'content' => [
        'value' => $row[4],
        'format' => 'full_html',
      ],
    ];
    $this->states[$row[1]] = $state;
  }

  protected function mapDataToRegion(array $row): void {
    if ($row[1] === 'unassigned') {
      return;
    }

    $action = match($row[8]) {
      'info' => 'content',
      'link' => 'url',
      default => 'none',
    };

    $config = $this->config('simple_interactive_maps.settings');

    $id = $this->convertNameToMachineName($row[1]);
    $region = [
      'id' => $id,
      'label' => $row[1],
      'override_colors' => $row[14] === 'yes',
      'fill_color' => $row[10],
      'hover_color' => $row[11],
      'stroke_color' => $config->get('default_stroke_color'),
      'override_tooltip' => $row[11] === 'yes',
      'tooltip' => [
        'value' => $config->get('default_state_tooltip')['value'],
        'format' => $config->get('default_state_tooltip')['format'],
      ],
      'override_action' => $row[12] === 'yes',
      'action' => $action,
      'url' => $row[10],
      'content' => [
        'value' => $row[11],
        'format' => $row[12],
      ],
    ];
    $this->regions[$id] = $region;
    $this->regionMap[$row[0]] = $id;
  }

  protected function convertNameToMachineName(string $name): string {
    $name = strtolower($name);
    $name = preg_replace('/[^a-z0-9_ ]/', '', strtolower($name));
    return str_replace(' ', '_', $name);
  }

  protected function getUploadFieldId(): string {
    return 'wordpress_data_file';
  }

}
