<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps;

/**
 * Load map data from the source data file.
 */
final class MapDataLoader {

  /**
   * The path to the map data file.
   */
  private const MAP_DATA_FILE_PATH = __DIR__ . '/../data/us-map-data.json';

  /**
   * The key for the regions data in the temp store.
   */
  public const MAP_TEMP_STORE_REGIONS_KEY = 'regions';

  /**
   * The key for the state details data in the temp store.
   */
  public const MAP_TEMP_STORE_DETAILS_KEY = 'states';

  /**
   * Load the map data into an array.
   *
   * @return array
   *   The map data.
   *
   * @throws \JsonException
   */
  public function loadMapData(InteractiveMapInterface $map = NULL): array {
    // Load map data from file.
    $data = $this->loadRawMapData();

    $json = json_decode($data, TRUE, 512, JSON_THROW_ON_ERROR);

    if ($map instanceof InteractiveMapInterface) {
      $this->mergeConfigData($json, $map);
    }

    return $json;
  }

  private function mergeConfigData(array &$mapData, InteractiveMapInterface $map): void {
    $details = $map->get('details');
    $regions = $map->get('regions');

    array_walk($mapData, function (&$item, $key) use ($details, $regions) {
      if (array_key_exists($key, $details)) {
        $item = array_merge($item, $details[$key]);
        if (($item['region'] !== '') && $regions[$item['region']]) {
          $this->applyRegionOverrides($item, $regions[$item['region']]);
        }
        $item['tooltip']['processed'] = is_array($item['tooltip']) ? check_markup($item['tooltip']['value'], $item['tooltip']['format']) : '';
        $item['content']['processed'] = is_array($item['content']) ? check_markup($item['content']['value'], $item['content']['format']) : '';
      }
    });
  }

  private function applyRegionOverrides(array &$item, array $region): void {
    if ($region['override_colors']) {
      $item['fill_color'] = $region['fill_color'];
      $item['hover_color'] = $region['hover_color'];
      $item['stroke_color'] = $region['stroke_color'];
    }

    if ($region['override_tooltip']) {
      $item['tooltip'] = $region['tooltip'] ?? ['value' => '', 'format' => 'plain_text'];
    }

    if ($region['override_action']) {
      $item['action'] = $region['action'];
      $item['url'] = $region['url'];
      $item['content'] = $region['content'];
    }
  }

  /**
   * Load the raw map data as a string.
   *
   * @return string
   *   The raw map data.
   */
  public function loadRawMapData(): string {
    return file_get_contents(self::MAP_DATA_FILE_PATH);
  }

}
